import gtk
import commands



#
# Makes the given window skip the taskbar (at least under GNOME).
#
def skip_taskbar_and_pager(window):

    # TODO: this is a hack and should be replaced as soon as PyGTK supports
    #       'set_skip_taskbar_hint()' and 'set_skip_pager_hint()'

    try:
        fail, lines = commands.getstatusoutput("xwininfo -root -children -int "
                                               "|grep \"gDeskCal\"")
        lines = lines.splitlines()
        xids = []
        for l in lines:
            xids.append(int(l.split()[0]))

        for xid in xids:
            fail, nil = commands.getstatusoutput("xprop -id \"" + str(xid) +
                                                 "\" -f _NET_WM_STATE 32a "
                                                 "-set _NET_WM_STATE "
                                                 "_NET_WM_STATE_SKIP_TASKBAR")
    except:
        pass

    # doesn't work at all
    #window.window.property_change("_NET_WM_STATE",
    #                              "ATOM",
    #                              32,
    #                              gtk.gdk.PROP_MODE_APPEND,
    #                              ("_NET_WM_STATE_SKIP_TASKBAR",))


#
# Restricts the given coordinates to visible values.
#
def _crop_coords(x, y, width, height):

    scrwidth = gtk.gdk.screen_width()
    scrheight = gtk.gdk.screen_height()

    x = min(max(0, x), scrwidth - 1)
    y = min(max(0, y), scrheight - 1)
    width = min(scrwidth - x, width)
    height = min(scrheight - y, height)

    return (x, y, width, height)



#
# Captures the wallpaper image by making a screen shot.
#
def get_wallpaper_fallback(x, y, width, height):

    x, y, width, height = _crop_coords(x, y, width, height)
    
    rootwin = gtk.gdk.get_default_root_window()
    pbuf = gtk.gdk.Pixbuf(0, 1, 8, width, height)
    pbuf.get_from_drawable(rootwin, rootwin.get_colormap(),
                           x, y, 0, 0, width, height)

    return pbuf



#
# Captures the wallpaper image by accessing the background pixmap.
#
def get_wallpaper(x, y, width, height):

    x, y, width, height = _crop_coords(x, y, width, height)

    # get wallpaper
    pmap_id = get_wallpaper_id()
    pmap = gtk.gdk.gdk_pixmap_foreign_new(pmap_id)
    pwidth, pheight = pmap.get_size()

    # create pixbuf
    rootwin = gtk.gdk.get_default_root_window()
    pbuf = gtk.gdk.Pixbuf(0, 1, 8, width, height)


    # tile wallpaper over pixbuf
    sx = -(x % pwidth)
    sy = -(y % pheight)
    for x in xrange(sx, width, pwidth):
        for y in xrange(sy, height, pheight):
            dstx = max(0, x)
            dsty = max(0, y)
            srcx = dstx - x
            srcy = dsty - y

            w = min(pwidth - srcx, width - dstx)
            h = min(pheight - srcy, height - dsty)

            pbuf.get_from_drawable(pmap, rootwin.get_colormap(),
                                   srcx, srcy, dstx, dsty, w, h)
        #end for
    #end for

    return pbuf




#
# Returns the ID of the background pixmap.
#
def get_wallpaper_id():

    try:
        root = gtk.gdk.get_default_root_window()
        id = root.property_get("_XROOTPMAP_ID")[2][0]
        return long(id)

    except:
        raise NotImplementedError
